import org.jetel.exception.ComponentNotReadyException;
import org.jetel.exception.TransformException;
import org.jetel.util.file.FileUtils;
import org.jetel.util.string.StringUtils;
import org.python.core.PyException;
import org.python.util.PythonInterpreter;

public final class PythonBridge extends org.jetel.component.CTLRecordTransform {

	private PythonInterpreter interp;
	
	//initialize interpreter
    public final void globalScopeInit() throws ComponentNotReadyException {   	
    	interp = new PythonInterpreter();
    	
    	//import utility methods
    	executeFile("./python/clover_utils.py");
    	
    	//read PythonScriptURL parameter
    	String pathURL = getGraph().getGraphParameters().getGraphParameter("PythonScriptURL").getValue();
    	
    	//read PythonScript parameter
    	String script = getGraph().getGraphParameters().getGraphParameter("PythonScript").getValue();
    	
    	//check whether URL or script is present
    	if (!(StringUtils.isEmpty(pathURL) ^ StringUtils.isEmpty(script))) {
    		throw new ComponentNotReadyException("Please define either 'Python script URL' or 'Python script' property.");
    	}
    	
    	//execute script on URL
    	if (!StringUtils.isEmpty(pathURL)) {
    		executeFile(pathURL);
    	//execute script code
    	} else if (!StringUtils.isEmpty(script)) {
    		interp.exec(script);
    	}
    }
    
    //execute script on given URL relative to project context
    private void executeFile(String path) {
    	//convert path to absolute
    	String absolutePath = FileUtils.getJavaFile(getGraph().getRuntimeContext().getContextURL(), path).getAbsolutePath(); 

    	//execute script which should not process anything but define method transform()
    	interp.execfile(absolutePath);
	}

	//release interpreter
    @Override
    public void postExecute() throws ComponentNotReadyException {
    	super.postExecute();
    	
    	//cleanup assure memory release
    	if (interp!=null) {
    		interp.cleanup();
    	}
    }

    //transform single record
    protected final java.lang.Integer transformDelegate() throws org.jetel.exception.ComponentNotReadyException, org.jetel.exception.TransformException {
    	//set current input record, output record and logger into Python global variables
    	interp.set("input_record", getInputRecord(0));
    	interp.set("output_record", getOutputRecord(0));
    	interp.set("logger", getNode().getLog());
  
    	//execute transformation
    	try {
			interp.exec("transform()");
		} catch (PyException e) {
			String ex = e.toString();
			if (ex.contains("DataRecordImpl.getField")) {
				ex = "\n\nHINT: It seems you're trying to access non-existent field from your Python script\n\n" + ex;
			}
			throw new TransformException("Error in Python script: " + ex, e);
		}
    	
    	//return ALL;
        return 2147483647;
    }
}
